@@ -1,12 +1,12 @@
 // Licensed under the Apache License, Version 2.0 (the "License"); you may not
-// use this file except in compliance with the License.  You may obtain a copy
-// of the License at
+// use this file except in compliance with the License. You may obtain a copy of
+// the License at
 //
 //   http://www.apache.org/licenses/LICENSE-2.0
 //
 // Unless required by applicable law or agreed to in writing, software
 // distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
-// WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the
+// WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 // License for the specific language governing permissions and limitations under
 // the License.
 
@@ -49,7 +49,7 @@
             var dbsOnPage = dbs.slice(offset, offset + maxPerPage);
 
             $.each(dbsOnPage, function(idx, dbName) {
-              $("#databases tbody.content").append("<tr>" + 
+              $("#databases tbody.content").append("<tr>" +
                 "<th><a href='database.html?" + encodeURIComponent(dbName) + "'>" +
                   dbName + "</a></th>" +
                 "<td class='size'></td><td class='count'></td>" +
@@ -97,14 +97,13 @@
       var dbName = decodeURIComponent(urlParts.shift());
       var viewName = (urlParts.length > 0) ? urlParts.join("/") : null;
       if (viewName) {
-        viewName = decodeURIComponent(viewName);
         $.cookies.set(dbName + ".view", viewName);
       } else {
         viewName = $.cookies.get(dbName + ".view", "");
         if (viewName) {
           this.redirecting = true;
           location.href = "database.html?" + encodeURIComponent(dbName) +
-            "/" + encodeDocId(viewName);
+            "/" + viewName;
         }
       }
       var db = $.couch.db(dbName);
@@ -123,20 +122,8 @@
         ruby: "{|doc|\n  emit(nil, doc);\n}"
       }
 
-      this.addDocument = function() {
-        $.showDialog("dialog/_create_document.html", {
-          submit: function(data, callback) {
-            db.saveDoc(data.docid ? {_id: data.docid} : {}, {
-              error: function(status, error, reason) {
-                callback({docid: reason});
-              },
-              success: function(resp) {
-                location.href = "document.html?" + encodeURIComponent(dbName) +
-                                "/" + encodeDocId(resp.id);
-              }
-            });
-          }
-        });
+      this.newDocument = function() {
+        location.href = "document.html?" + encodeURIComponent(db.name);
       }
 
       this.compactDatabase = function() {
@@ -198,6 +185,7 @@
                                      .bind("textInput", updateDirtyState);
             }
             $("#language").change(updateDirtyState);
+            page.updateDocumentListing();
           });
           $("#grouptruenotice").show();
         } else if (viewName == "_temp_view") {
@@ -206,7 +194,9 @@
             $.cookies.get(db.name + ".map", templates[page.viewLanguage]),
             $.cookies.get(db.name + ".reduce", "")
           );
-        $("#grouptruenotice").show();
+          $("#grouptruenotice").show();
+        } else {
+          page.updateDocumentListing();
         }
         page.populateLanguagesMenu();
         if (this.isTempView) {
@@ -216,32 +206,48 @@
 
       // Populate the languages dropdown, and listen to selection changes
       this.populateLanguagesMenu = function() {
+        var all_langs = {};
+        fill_language = function() {
+          var select = $("#language");
+          for (var language in all_langs) {
+            var option = $(document.createElement("option"))
+              .attr("value", language).text(language)
+              .appendTo(select);
+          }
+          if (select[0].options.length == 1) {
+            select[0].disabled = true;
+          } else {
+            select[0].disabled = false;
+            select.val(page.viewLanguage);
+            select.change(function() {
+              var language = $("#language").val();
+              if (language != page.viewLanguage) {
+                var mapFun = $("#viewcode_map").val();
+                if (mapFun == "" || mapFun == templates[page.viewLanguage]) {
+                  // no edits made, so change to the new default
+                  $("#viewcode_map").val(templates[language]);
+                }
+                page.viewLanguage = language;
+                $("#viewcode_map")[0].focus();
+              }
+              return false;
+            });
+          }
+        }
         $.couch.config({
           success: function(resp) {
-            var select = $("#language");
             for (var language in resp) {
-              var option = $(document.createElement("option"))
-                .attr("value", language).text(language)
-                .appendTo(select);
+              all_langs[language] = resp[language];
             }
-            if (select[0].options.length == 1) {
-              select[0].disabled = true;
-            } else {
-              select.val(page.viewLanguage);
-              select.change(function() {
-                var language = $("#language").val();
-                if (language != page.viewLanguage) {
-                  var mapFun = $("#viewcode_map").val();
-                  if (mapFun == "" || mapFun == templates[page.viewLanguage]) {
-                    // no edits made, so change to the new default
-                    $("#viewcode_map").val(templates[language]);
-                  }
-                  page.viewLanguage = language;
-                  $("#viewcode_map")[0].focus();
+
+            $.couch.config({
+              success: function(resp) {
+                for (var language in resp) {
+                  all_langs[language] = resp[language];
                 }
-                return false;
-              });
-            }
+                fill_language();
+              }
+            }, "native_query_servers");
           }
         }, "query_servers");
       }
@@ -249,24 +255,27 @@
       this.populateViewsMenu = function() {
         var select = $("#switch select");
         db.allDocs({startkey: "_design/", endkey: "_design0",
+          include_docs: true,
           success: function(resp) {
             select[0].options.length = 3;
             for (var i = 0; i < resp.rows.length; i++) {
-              db.openDoc(resp.rows[i].id, {
-                success: function(doc) {
-                  var optGroup = $(document.createElement("optgroup"))
-                    .attr("label", doc._id.substr(8));
-                  for (var name in doc.views) {
-                    var option = $(document.createElement("option"))
-                      .attr("value", doc._id + "/" + name).text(name)
-                      .appendTo(optGroup);
-                    if (doc._id + "/" + name == viewName) {
-                      option[0].selected = true;
-                    }
-                  }
-                  optGroup.appendTo(select);
+              var doc = resp.rows[i].doc;
+              var optGroup = $(document.createElement("optgroup"))
+                .attr("label", doc._id.substr(8)).appendTo(select);
+              var viewNames = [];
+              for (var name in doc.views) {
+                viewNames.push(name);
+              }
+              viewNames.sort();
+              for (var j = 0; j < viewNames.length; j++) {
+                var path = $.couch.encodeDocId(doc._id) + "/_view/" +
+                  encodeURIComponent(viewNames[j]);
+                var option = $(document.createElement("option"))
+                  .attr("value", path).text(viewNames[j]).appendTo(optGroup);
+                if (path == viewName) {
+                  option[0].selected = true;
                 }
-              });
+              }
             }
           }
         });
@@ -282,9 +291,9 @@
       this.revertViewChanges = function(callback) {
         if (!page.storedViewCode) {
           var viewNameParts = viewName.split("/");
-          var designDocId = viewNameParts[1];
-          var localViewName = viewNameParts[2];
-          db.openDoc(["_design", designDocId].join("/"), {
+          var designDocId = decodeURIComponent(viewNameParts[1]);
+          var localViewName = decodeURIComponent(viewNameParts[3]);
+          db.openDoc("_design/" + designDocId, {
             error: function(status, error, reason) {
               if (status == 404) {
                 $.cookies.remove(dbName + ".view");
@@ -292,6 +301,10 @@
               }
             },
             success: function(resp) {
+              if(!resp.views || !resp.views[localViewName]) {
+                $.cookies.remove(dbName + ".view");
+                location.href = "database.html?" + encodeURIComponent(db.name);
+              }
               var viewCode = resp.views[localViewName];
               page.viewLanguage = resp.language || "javascript";
               $("#language").val(page.viewLanguage);
@@ -327,8 +340,8 @@
       this.saveViewAs = function() {
         if (viewName && /^_design/.test(viewName)) {
           var viewNameParts = viewName.split("/");
-          var designDocId = viewNameParts[1];
-          var localViewName = viewNameParts[2];
+          var designDocId = decodeURIComponent(viewNameParts[1]);
+          var localViewName = decodeURIComponent(viewNameParts[3]);
         } else {
           var designDocId = "", localViewName = "";
         }
@@ -402,8 +415,8 @@
                     callback();
                     page.isDirty = false;
                     location.href = "database.html?" + encodeURIComponent(dbName) +
-                      "/" + encodeDocId(doc._id) +
-                      "/" + encodeURIComponent(data.name);
+                      "/" + $.couch.encodeDocId(doc._id) +
+                      "/_view/" + encodeURIComponent(data.name);
                   }
                 });
               }
@@ -423,9 +436,9 @@
 
       this.saveViewChanges = function() {
         var viewNameParts = viewName.split("/");
-        var designDocId = viewNameParts[1];
-        var localViewName = viewNameParts[2];
-        db.openDoc(["_design", designDocId].join("/"), {
+        var designDocId = decodeURIComponent(viewNameParts[1]);
+        var localViewName = decodeURIComponent(viewNameParts[3]);
+        db.openDoc("_design/" + designDocId, {
           success: function(doc) {
             var numViews = 0;
             for (var viewName in (doc.views || {})) {
@@ -454,14 +467,21 @@
 
       this.updateDesignDocLink = function() {
         if (viewName && /^_design/.test(viewName)) {
-          var docId = "_design/" + viewName.split("/")[1];
+          var docId = "_design/" + decodeURIComponent(viewName.split("/")[1]);
           $("#designdoc-link").attr("href", "document.html?" +
-            encodeURIComponent(dbName) + "/" + encodeDocId(docId)).text(docId);
+            encodeURIComponent(dbName) + "/" + $.couch.encodeDocId(docId)).text(docId);
         } else {
           $("#designdoc-link").removeAttr("href").text("");
         }
       }
 
+      this.jumpToDocument = function(docId) {
+        if (docId != "") {
+          location.href = 'document.html?' + encodeURIComponent(db.name)
+            + "/" + $.couch.encodeDocId(docId);
+        }
+      }
+
       this.updateDocumentListing = function(options) {
         if (options === undefined) options = {};
         if (options.limit === undefined) {
@@ -470,7 +490,7 @@
           // reduce views
           options.limit = per_page + 1;
         } else {
-          per_page = options.limit;
+          per_page = options.limit - 1;
         }
         if ($("#documents thead th.key").is(".desc")) {
           if (typeof options.descending == 'undefined') options.descending = true;
@@ -489,11 +509,14 @@
             resp.offset = 0;
           }
           var descending_reverse = ((options.descending && !descend) || (descend && (options.descending === false)));
+          var has_reduce_prev = resp.total_rows === undefined && (descending_reverse ? resp.rows.length > per_page : options.startkey !== undefined);
           if (descending_reverse && resp.rows) {
             resp.rows = resp.rows.reverse();
+            if (resp.rows.length > per_page) {
+              resp.rows.push(resp.rows.shift());
+            }
           }
-          var has_reduce_prev = resp.total_rows === undefined && (descending_reverse ? resp.rows.length > per_page : options.startkey !== undefined);
-          if (resp.rows !== null && (has_reduce_prev || (descending_reverse ? 
+          if (resp.rows !== null && (has_reduce_prev || (descending_reverse ?
             (resp.total_rows - resp.offset > per_page) :
             (resp.offset > 0)))) {
             $("#paging a.prev").attr("href", "#" + (resp.offset - per_page)).click(function() {
@@ -516,8 +539,8 @@
             $("#paging a.prev").removeAttr("href");
           }
           var has_reduce_next = resp.total_rows === undefined && (descending_reverse ? options.startkey !== undefined : resp.rows.length > per_page);
-          if (resp.rows !== null && (has_reduce_next || (descending_reverse ? 
-            (resp.offset - resp.total_rows < per_page) : 
+          if (resp.rows !== null && (has_reduce_next || (descending_reverse ?
+            (resp.offset - resp.total_rows < per_page) :
             (resp.total_rows - resp.offset > per_page)))) {
             $("#paging a.next").attr("href", "#" + (resp.offset + per_page)).click(function() {
               var opt = {
@@ -548,7 +571,7 @@
             }
             if (row.id) {
               $("<td class='key'><a href='document.html?" + encodeURIComponent(db.name) +
-                "/" + encodeDocId(row.id) + "'><strong></strong><br>" +
+                "/" + $.couch.encodeDocId(row.id) + "'><strong></strong><br>" +
                 "<span class='docid'>ID:&nbsp;" + row.id + "</span></a></td>")
                 .find("strong").text(key).end()
                 .appendTo(tr);
@@ -601,8 +624,8 @@
             $("#viewcode").show().removeClass("collapsed");
             var mapFun = $("#viewcode_map").val();
             $.cookies.set(db.name + ".map", mapFun);
-            var reduceFun = $("#viewcode_reduce").val() || null;
-            if (reduceFun != null) {
+            var reduceFun = $.trim($("#viewcode_reduce").val()) || null;
+            if (reduceFun) {
               $.cookies.set(db.name + ".reduce", reduceFun);
               options.group = true;
             } else {
@@ -617,14 +640,15 @@
           } else {
             $("#viewcode").show();
             var currentMapCode = $("#viewcode_map").val();
-            var currentReduceCode = $("#viewcode_reduce").val() || null;
+            var currentReduceCode = $.trim($("#viewcode_reduce").val()) || null;
             if (currentReduceCode) {
               options.group = true;
             }
             if (page.isDirty) {
               db.query(currentMapCode, currentReduceCode, page.viewLanguage, options);
             } else {
-              db.view(viewName.substr(8), options);
+              var viewParts = viewName.split('/');
+              db.view(viewParts[1] + "/" + viewParts[3], options);
             }
           }
         }
@@ -644,19 +668,29 @@
     CouchDocumentPage: function() {
       var urlParts = location.search.substr(1).split("/");
       var dbName = decodeURIComponent(urlParts.shift());
-      var idParts = urlParts.join("/").split("@", 2);
-      var docId = decodeURIComponent(idParts[0]);
-      var docRev = (idParts.length > 1) ? idParts[1] : null;
+      if (urlParts.length) {
+        var idParts = urlParts.join("/").split("@", 2);
+        var docId = decodeURIComponent(idParts[0]);
+        var docRev = (idParts.length > 1) ? idParts[1] : null;
+        this.isNew = false;
+      } else {
+        var docId = $.couch.newUUID();
+        var docRev = null;
+        this.isNew = true;
+      }
       var db = $.couch.db(dbName);
 
       this.dbName = dbName;
       this.db = db;
       this.docId = docId;
       this.doc = null;
-      this.isDirty = false;
+      this.isDirty = this.isNew;
       page = this;
 
       this.activateTabularView = function() {
+        if ($("#fields tbody.source textarea").length > 0)
+          return;
+
         $("#tabs li").removeClass("active").filter(".tabular").addClass("active");
         $("#fields thead th:first").text("Field").attr("colspan", 1).next().show();
         $("#fields tbody.content").show();
@@ -668,7 +702,41 @@
         $("#fields thead th:first").text("Source").attr("colspan", 2).next().hide();
         $("#fields tbody.content").hide();
         $("#fields tbody.source").find("td").each(function() {
-          $(this).html($("<pre></pre>").html($.futon.formatJSON(page.doc, {html: true})));
+          $(this).html($("<pre></pre>").html($.futon.formatJSON(page.doc, {html: true})))
+            .makeEditable({allowEmpty: false,
+              createInput: function(value) {
+                return $("<textarea rows='8' cols='80' spellcheck='false'></textarea>");
+              },
+              prepareInput: function(input) {
+                $(input).makeResizable({vertical: true});
+              },
+              end: function() {
+                $(this).html($("<pre></pre>").html($.futon.formatJSON(page.doc, {html: true})));
+              },
+              accept: function(newValue) {
+                page.doc = JSON.parse(newValue);
+                page.isDirty = true;
+                page.updateFieldListing(true);
+              },
+              populate: function(value) {
+                return $.futon.formatJSON(page.doc);
+              },
+              validate: function(value) {
+                try {
+                  var doc = JSON.parse(value);
+                  if (typeof doc != "object")
+                    throw new SyntaxError("Please enter a valid JSON document (for example, {}).");
+                  return true;
+                } catch (err) {
+                  var msg = err.message;
+                  if (msg == "parseJSON" || msg == "JSON.parse") { 
+                    msg = "There is a syntax error in the document.";
+                  }
+                  $("<div class='error'></div>").text(msg).appendTo(this);
+                  return false;
+                }
+              }
+            });
         }).end().show();
       }
 
@@ -701,7 +769,7 @@
         }
       }
 
-      this.updateFieldListing = function() {
+      this.updateFieldListing = function(noReload) {
         $("#fields tbody.content").empty();
 
         function handleResult(doc, revs) {
@@ -726,42 +794,52 @@
             if (currentIndex < revs.length - 1) {
               var prevRev = revs[currentIndex + 1].rev;
               $("#paging a.prev").attr("href", "?" + encodeURIComponent(dbName) +
-                "/" + encodeDocId(docId) + "@" + prevRev);
+                "/" + $.couch.encodeDocId(docId) + "@" + prevRev);
             }
             if (currentIndex > 0) {
               var nextRev = revs[currentIndex - 1].rev;
               $("#paging a.next").attr("href", "?" + encodeURIComponent(dbName) +
-                "/" + encodeDocId(docId) + "@" + nextRev);
+                "/" + $.couch.encodeDocId(docId) + "@" + nextRev);
             }
             $("#fields tbody.footer td span").text("Showing revision " +
               (revs.length - currentIndex) + " of " + revs.length);
           }
-          if (location.hash == "#source") {
+          if (location.hash == "#source" && !noReload) {
             page.activateSourceView();
           }
         }
 
-        db.openDoc(docId, {revs_info: true,
-          success: function(doc) {
-            var revs = doc._revs_info;
-            delete doc._revs_info;
-            if (docRev != null) {
-              db.openDoc(docId, {rev: docRev,
-                error: function(status, error, reason) {
-                  alert("The requested revision was not found. " +
-                        "You will be redirected back to the latest revision.");
-                  location.href = "?" + encodeURIComponent(dbName) +
-                    "/" + encodeDocId(docId);
-                },
-                success: function(doc) {
-                  handleResult(doc, revs);
-                }
-              });
-            } else {
-              handleResult(doc, revs);
+        if (noReload) {
+          handleResult(page.doc, []);
+          return;
+        }
+
+        if (!page.isNew) {
+          db.openDoc(docId, {revs_info: true,
+            success: function(doc) {
+              var revs = doc._revs_info || [];
+              delete doc._revs_info;
+              if (docRev != null) {
+                db.openDoc(docId, {rev: docRev,
+                  error: function(status, error, reason) {
+                    alert("The requested revision was not found. You will " +
+                          "be redirected back to the latest revision.");
+                    location.href = "?" + encodeURIComponent(dbName) +
+                      "/" + $.couch.encodeDocId(docId);
+                  },
+                  success: function(doc) {
+                    handleResult(doc, revs);
+                  }
+                });
+              } else {
+                handleResult(doc, revs);
+              }
             }
-          }
-        });
+          });
+        } else {
+          handleResult({_id: docId}, []);
+          $("#fields tbody td").dblclick();
+        }
       }
 
       this.deleteDocument = function() {
@@ -785,7 +863,7 @@
           success: function(resp) {
             page.isDirty = false;
             location.href = "?" + encodeURIComponent(dbName) +
-              "/" + encodeDocId(docId);
+              "/" + $.couch.encodeDocId(page.docId);
           }
         });
       }
@@ -808,12 +886,12 @@
             var form = $("#upload-form");
             form.find("#progress").css("visibility", "visible");
             form.ajaxSubmit({
-              url: db.uri + encodeDocId(page.docId),
+              url: db.uri + $.couch.encodeDocId(page.docId),
               success: function(resp) {
                 form.find("#progress").css("visibility", "hidden");
                 page.isDirty = false;
                 location.href = "?" + encodeURIComponent(dbName) +
-                  "/" + encodeDocId(docId);
+                  "/" + $.couch.encodeDocId(page.docId);
               }
             });
           }
@@ -874,12 +952,21 @@
             if (keyCode == 9) { // tab, move to editing the value
               row.find("td").dblclick();
             }
+          },
+          validate: function(newName, oldName) {
+            $("div.error", this).remove();
+            if (newName != oldName && doc[newName] !== undefined) {
+              $("<div class='error'>Already have field with that name.</div>")
+                .appendTo(this);
+              return false;
+            }
+            return true;
           }
         });
       }
 
       function _initValue(doc, row, fieldName) {
-        if (fieldName == "_id" || fieldName == "_rev") {
+        if ((fieldName == "_id" && !page.isNew) || fieldName == "_rev") {
           return;
         }
 
@@ -900,19 +987,30 @@
             }
           },
           accept: function(newValue) {
-            doc[row.data("name")] = JSON.parse(newValue);
+            var fieldName = row.data("name");
+            doc[fieldName] = JSON.parse(newValue);
             page.isDirty = true;
+            if (fieldName == "_id") {
+              page.docId = page.doc._id = doc[fieldName];
+              $("h1 strong").text(page.docId);
+            }
           },
           populate: function(value) {
             return $.futon.formatJSON(doc[row.data("name")]);
           },
           validate: function(value) {
+            $("div.error", this).remove();
             try {
-              JSON.parse(value);
+              var parsed = JSON.parse(value);
+              if (row.data("name") == "_id" && typeof(parsed) != "string") {
+                $("<div class='error'>The document ID must be a string.</div>")
+                  .appendTo(this);
+                return false;
+              }
               return true;
             } catch (err) {
               var msg = err.message;
-              if (msg == "parseJSON") {
+              if (msg == "parseJSON" || msg == "JSON.parse") {
                 msg = "Please enter a valid JSON value (for example, \"string\").";
               }
               $("<div class='error'></div>").text(msg).appendTo(this);
@@ -956,16 +1054,16 @@
       }
 
       function _renderAttachmentItem(name, attachment) {
-        var attachmentHref = db.uri + encodeDocId(docId) 
+        var attachmentHref = db.uri + $.couch.encodeDocId(page.docId)
           + "/" + encodeAttachment(name);
         var li = $("<li></li>");
         $("<a href='' title='Download file' target='_top'></a>").text(name)
           .attr("href", attachmentHref)
           .wrapInner("<tt></tt>").appendTo(li);
-        $("<span>()</span>").text("" + $.futon.formatSize(attachment.length) + 
+        $("<span>()</span>").text("" + $.futon.formatSize(attachment.length) +
           ", " + attachment.content_type).addClass("info").appendTo(li);
         if (name == "tests.js") {
-          li.find('span.info').append(', <a href="/_utils/couch_tests.html?' 
+          li.find('span.info').append(', <a href="/_utils/couch_tests.html?'
             + attachmentHref + '">open in test runner</a>');
         }
         _initAttachmentItem(name, attachment, li);
@@ -990,17 +1088,6 @@
 
   });
 
-  function encodeDocId(docid) {
-    var encoded, parts = docid.split('/');
-    if (parts[0] == '_design') {
-      parts.shift();
-      encoded = encodeURIComponent(parts.join('/'));
-      return '_design/' + encoded;
-    } else {
-      return encodeURIComponent(docid);
-    }
-  };
-
   function encodeAttachment(name) {
     var encoded = [], parts = name.split('/');
     for (var i=0; i < parts.length; i++) {
